@@ -3,6 +3,8 @@ package Dist::Requires;
 # ABSTRACT: Identify requirements for a distribution
 
 use Moose;
+use Moose::Util::TypeConstraints;
+
 use Carp;
 use CPAN::Meta;
 use Module::CoreList;
@@ -23,14 +25,22 @@ use namespace::autoclean;
 
 #-----------------------------------------------------------------------------
 
-our $VERSION = '0.002'; # VERSION
+our $VERSION = '0.003'; # VERSION
+
+#-----------------------------------------------------------------------------
+# Some custom types
+
+class_type 'Version', { class => 'version' };
+coerce 'Version', from 'Str', via { version->parse($_) };
+coerce 'Version', from 'Num', via { version->parse($_) };
+# TODO: Put those on CPAN as MooseX::Types::Version
 
 #-----------------------------------------------------------------------------
 
 
 has target_perl => (
     is       => 'ro',
-    isa      => 'Str',
+    isa      => 'Str',  # TODO: make this a Path::Class::File
     default  => $^X,
     init_arg => undef,
 );
@@ -38,8 +48,10 @@ has target_perl => (
 
 has target_perl_version => (
     is         => 'ro',
-    isa        => 'Num',
-    default    => $],
+    isa        => 'Version',
+    coerce     => 1,
+    lazy       => 1,
+    default    => sub { version->parse( $] ) },
     # TODO: lazy_build => 1,
 );
 
@@ -66,9 +78,9 @@ has filter => (
 sub _build_filter {
     my ($self) = @_;
 
-    my $v = $self->target_perl_version();
-    my $core_packages = $Module::CoreList::version{$v};  ## no critic (PackageVar)
-    croak "No such perl version: $v" if not $core_packages;
+    # version.pm doesn't always strip trailing zeros
+    my $tpv           = $self->target_perl_version->numify() + 0;
+    my $core_packages = $Module::CoreList::version{$tpv};  ## no critic (PackageVar)
 
     return { __versionize_values( %{$core_packages} ) };
 }
@@ -87,6 +99,23 @@ sub _build_target_perl_version {
 
 #-----------------------------------------------------------------------------
 
+sub BUILD {
+    my ($self) = @_;
+
+    my $tpv = $self->target_perl_version()->numify();
+    $tpv += 0;  # version.pm doesn't always strip trailing zeros
+
+    croak "The target_perl_version ($tpv) cannot be greater than this perl ($])"
+        if $tpv > $];
+
+    croak "Unknown version of perl: $tpv"
+        if not exists $Module::CoreList::version{$tpv};  ## no critic (PackageVar)
+
+    return $self;
+}
+
+#-----------------------------------------------------------------------------
+
 
 sub requires {
     my ( $self, %args ) = @_;
@@ -119,8 +148,16 @@ sub _unpack_dist {
     my $ae = Archive::Extract->new( archive => $dist );
     $ae->extract( to => $temp ) or croak $ae->error();
 
-    my $dist_root = $temp->subdir( ( @{$ae->files()} )[0] );
-    croak "$dist did not unpack cleanly into a directory" if not -d $dist_root;
+    # Originally, we just returned the first entry in $ae->files() as the
+    # $dist_root, but that proved to be unreliable.  Better to actually look
+    # in $temp and see what is there.  For a well-packaged archive, $temp
+    # should contain exactly one child and that child should be a directory.
+
+    my @children = $temp->children();
+    croak "$dist did not unpack into a single directory" if @children != 1;
+
+    my $dist_root = $children[0];
+    croak "$dist did not unpack into a directory" if not -d $dist_root;
 
     return $dist_root;
 }
@@ -150,18 +187,17 @@ sub _configure {
 
     my $try_eumm = sub {
         if ( -e 'Makefile.PL' ) {
-            return $self->_run( [$self->target_perl(), "Makefile.PL"] ) && -e 'Makefile';
+            return $self->_run( [$self->target_perl(), 'Makefile.PL'] ) && -e 'Makefile';
         }
     };
 
 
     my $try_mb = sub {
         if ( -e 'Build.PL' ) {
-            return $self->_run( [$self->target_perl(), "Build.PL"] ) && -e 'Build';
+            return $self->_run( [$self->target_perl(), 'Build.PL'] ) && -e 'Build';
         }
     };
 
-
     my $ok = $try_mb->() || $try_eumm->() || croak "Failed to configure $dist_dir";
 
     Cwd::chdir($old_cwd) or croak "Unable to chdir to $old_cwd: $!";
@@ -237,9 +273,10 @@ sub _run {
     $ENV{PERL_MM_OPT} .= " INSTALLMAN1DIR=none INSTALLMAN3DIR=none";
 
     my ($in, $out);
-    return run( $cmd, \$in, \$out, \$out, timeout( $self->timeout() ) )
-        or croak "Configuration failed: $?\noutput was: $out";
+    my $ok = run( $cmd, \$in, \$out, \$out, timeout( $self->timeout() ) );
+    $ok or croak "Configuration failed: $out";
 
+    return $ok;
 }
 
 #-----------------------------------------------------------------------------
@@ -278,7 +315,7 @@ Dist::Requires - Identify requirements for a distribution
 
 =head1 VERSION
 
-version 0.002
+version 0.003
 
 =head1 SYNOPSIS
 
@@ -313,16 +350,17 @@ the object is immutable and all attributes are read-only.
 
 =head2 target_perl => $PATH
 
-Sets the perl executable that will be used to configure the distribution.
-Defaults to the perl that loaded this module.  NOTE: this attribute is
-not configurable at this time.
+The path to the perl executable that will be used to configure the
+distribution.  Defaults to the perl that loaded this module.  NOTE:
+this attribute is not configurable at this time.
 
 =head2 target_perl_version => $FLOAT
 
 The core module list for the specified perl version will be used to
 filter the requirements.  This only matters if you're using the
 default package filter.  Defaults to the version of the perl specified
-by the C<perl> attribute..
+by the C<perl> attribute.  Can be specified as a decimal number, a
+dotted version string, or a L<version> object.
 
 =head2 timeout => $INT
 
@@ -373,6 +411,8 @@ when you install a distribution.
 
 L<Module::Depends>
 
+=for Pod::Coverage BUILD
+
 =head1 SUPPORT
 
 =head2 Perldoc
